/*-----------------------------------------------------------------------------
Program: Boyer (toy theorem prover)
Author:  E. Tick (after Lisp by R. Boyer)
Date:    November 12 1985

Notes:
1. To run:
        ?- go(X).
where X is output execution time.

Modularized by: F. Bueno
-----------------------------------------------------------------------------*/

:- module(mboyer,[ tautology/1 ],[ assertions ]).

:- use_module(engine(basiccontrol),[fail/0,true/0]).
:- use_module(engine(term_basic),[(=)/2]).
:- use_module(rewrite).

:- entry tautology(X) : ground(X).

go(T) :- time(_), wff(X), tautology(X), time(T).

time(T) :- statistics(runtime,[_,T]).

wff(implies(and(implies(X,Y),
        and(implies(Y,Z),
            and(implies(Z,U),
            implies(U,W)))),
        implies(X,W))) :-
    X = f(plus(plus(a,b),plus(c,zero))),
    Y = f(times(times(a,b),plus(c,d))),
    Z = f(reverse(append(append(a,b),[]))),
    U = equal(plus(a,b),difference(x,y)),
    W = lessp(remainder(a,b),member(a,length(b))).

:- push_prolog_flag(multi_arity_warnings,off).

tautology(Wff) :-
    rewrite(Wff,NewWff),
    tautology(NewWff,[],[]).

% M.H. Implications changed to ! for right now: incorrect under Quintus!!!
tautology(Wff,Tlist,Flist) :-
    (truep(Wff,Tlist) -> true
    ;falsep(Wff,Flist) -> fail
    ;Wff = if(If,Then,Else) ->
        (truep(If,Tlist) -> tautology(Then,Tlist,Flist)
        ;falsep(If,Flist) -> tautology(Else,Tlist,Flist)
        ;tautology(Then,[If|Tlist],Flist),    % both must hold
         tautology(Else,Tlist,[If|Flist])
        )
    ),!.

:- pop_prolog_flag(multi_arity_warnings).

truep(t,_) :- !.
truep(Wff,Tlist) :- member(Wff,Tlist).

falsep(f,_) :- !.
falsep(Wff,Flist) :- member(Wff,Flist).

member(X,[X|_]) :- !.
member(X,[_|T]) :- member(X,T).


